@@ -11,6 +11,14 @@ from .base import BaseModelClient
 # Set up logging
 logger = logging.getLogger(__name__)
 
+# Custom exception for Ollama API errors
+class OllamaApiError(Exception):
+    """Exception raised for errors in the Ollama API."""
+    def __init__(self, message: str, status_code: Optional[int] = None):
+        self.message = message
+        self.status_code = status_code
+        super().__init__(self.message)
+
 class OllamaClient(BaseModelClient):
     def __init__(self):
         from ..config import OLLAMA_BASE_URL
@@ -23,8 +31,96 @@ class OllamaClient(BaseModelClient):
         # Track model loading state
         self._model_loading = False
         
+        # Track preloaded models and their last use timestamp
+        self._preloaded_models = {}
+        
+        # Default timeout values (in seconds)
+        self.DEFAULT_TIMEOUT = 30
+        self.MODEL_LOAD_TIMEOUT = 120
+        self.MODEL_PULL_TIMEOUT = 3600  # 1 hour for large models
+        
         # Path to the cached models file
         self.models_cache_path = Path(__file__).parent.parent / "data" / "ollama-models.json"
+        
+    def get_timeout_for_model(self, model_id: str, operation: str = "generate") -> int:
+        """
+        Calculate an appropriate timeout based on model size
+        
+        Parameters:
+        - model_id: The model identifier
+        - operation: The operation type ('generate', 'load', 'pull')
+        
+        Returns:
+        - Timeout in seconds
+        """
+        # Default timeouts by operation
+        default_timeouts = {
+            "generate": self.DEFAULT_TIMEOUT,      # 30s
+            "load": self.MODEL_LOAD_TIMEOUT,       # 2min 
+            "pull": self.MODEL_PULL_TIMEOUT,       # 1h
+            "list": 5,                             # 5s
+            "test": 2                              # 2s
+        }
+        
+        # Parameter size multipliers
+        size_multipliers = {
+            # For models < 3B
+            "1b": 0.5,
+            "2b": 0.7,
+            "3b": 1.0,
+            # For models 3B-10B
+            "5b": 1.2,
+            "6b": 1.3,
+            "7b": 1.5,
+            "8b": 1.7,
+            "9b": 1.8,
+            # For models 10B-20B
+            "13b": 2.0,
+            "14b": 2.0,
+            # For models 20B-50B
+            "27b": 3.0,
+            "34b": 3.5,
+            "40b": 4.0,
+            # For models 50B+
+            "70b": 5.0,
+            "80b": 6.0,
+            "100b": 7.0,
+            "400b": 10.0,
+            "405b": 10.0,
+        }
+        
+        # Get the base timeout for the operation
+        base_timeout = default_timeouts.get(operation, self.DEFAULT_TIMEOUT)
+        
+        # Try to determine the model size from the model ID
+        model_size = "7b"  # Default assumption is 7B parameters
+        model_lower = model_id.lower()
+        
+        # Check for size indicators in the model name
+        for size in size_multipliers.keys():
+            if size in model_lower:
+                model_size = size
+                break
+                
+        # If it's a known large model without size in name
+        if "llama3.1" in model_lower and not any(size in model_lower for size in size_multipliers.keys()):
+            model_size = "8b"  # Default for llama3.1 without size specified
+            
+        # For first generation after model selection, if preloaded, use shorter timeout
+        if operation == "generate" and model_id in self._preloaded_models:
+            # For preloaded models, use a shorter timeout
+            return max(int(base_timeout * 0.7), 20)  # Min 20 seconds
+        
+        # Calculate final timeout with multiplier
+        multiplier = size_multipliers.get(model_size, 1.0)
+        timeout = int(base_timeout * multiplier)
+        
+        # For pull operation, ensure we have a reasonable maximum
+        if operation == "pull":
+            return min(timeout, 7200)  # Max 2 hours
+            
+        logger.info(f"Calculated timeout for {model_id} ({operation}): {timeout}s (base: {base_timeout}s, multiplier: {multiplier})")
+        return timeout
 
     @classmethod
     async def create(cls) -> 'OllamaClient':
@@ -53,7 +149,29 @@ class OllamaClient(BaseModelClient):
             style_instructions = self._get_style_instructions(style)
             debug_log(f"Adding style instructions: {style_instructions[:50]}...")
             formatted_messages.append(style_instructions)
+            
+        # Special case for title generation - check if this is a title generation message
+        is_title_generation = False
+        for msg in messages:
+            if msg.get("role") == "system" and "generate a brief, descriptive title" in msg.get("content", "").lower():
+                is_title_generation = True
+                debug_log("Detected title generation prompt")
+                break
         
+        # For title generation, use a direct approach
+        if is_title_generation:
+            debug_log("Using specialized formatting for title generation")
+            # Find the user message containing the input for title generation
+            user_msg = next((msg for msg in messages if msg.get("role") == "user"), None)
+            if user_msg and "content" in user_msg:
+                # Create a direct prompt
+                prompt = "You must generate a short, descriptive title (maximum 40 characters) for this conversation. ONLY output the title with no additional text, no quotes, and no explanation. Do not start with phrases like 'Here's a title' or 'Title:'. RESPOND ONLY WITH THE TITLE TEXT for the following message:\n\n" + user_msg["content"]
+                debug_log(f"Created title generation prompt: {prompt[:100]}...")
+                return prompt
+            else:
+                debug_log("Could not find user message for title generation, using standard formatting")
+        
+        # Standard processing for normal chat messages
         # Add message content, preserving conversation flow
         for i, msg in enumerate(messages):
             try:
@@ -115,7 +233,7 @@ class OllamaClient(BaseModelClient):
             async with aiohttp.ClientSession() as session:
                 async with session.get(
                     f"{self.base_url}/api/tags",
-                    timeout=5,
+                    timeout=aiohttp.ClientTimeout(total=5),
                     headers={"Accept": "application/json"}
                 ) as response:
                     response.raise_for_status()
@@ -145,22 +263,22 @@ class OllamaClient(BaseModelClient):
                     logger.info(f"Found {len(models)} Ollama models")
                     return models
                     
-        except aiohttp.ClientConnectorError:
+        except aiohttp.ClientConnectorError as e:
             error_msg = f"Could not connect to Ollama server at {self.base_url}. Please ensure Ollama is running and the URL is correct."
             logger.error(error_msg)
-            raise Exception(error_msg)
-        except aiohttp.ClientTimeout:
+            raise Exception(error_msg) from e
+        except asyncio.TimeoutError as e:
             error_msg = f"Connection to Ollama server at {self.base_url} timed out after 5 seconds. The server might be busy or unresponsive."
             logger.error(error_msg)
-            raise Exception(error_msg)
+            raise Exception(error_msg) from e
         except aiohttp.ClientError as e:
             error_msg = f"Ollama API error: {str(e)}"
             logger.error(error_msg)
-            raise Exception(error_msg)
+            raise Exception(error_msg) from e
         except Exception as e:
             error_msg = f"Unexpected error getting models: {str(e)}"
             logger.error(error_msg)
-            raise Exception(error_msg)
+            raise Exception(error_msg) from e
             
     async def generate_completion(self, messages: List[Dict[str, str]],
                                 model: str,
@@ -177,6 +295,7 @@ class OllamaClient(BaseModelClient):
             try:
                 async with aiohttp.ClientSession() as session:
                     logger.debug(f"Sending request to {self.base_url}/api/generate")
+                    gen_timeout = self.get_timeout_for_model(model, "generate")
                     async with session.post(
                         f"{self.base_url}/api/generate",
                         json={
@@ -185,19 +304,23 @@ class OllamaClient(BaseModelClient):
                             "temperature": temperature,
                             "stream": False
                         },
-                        timeout=30
+                        timeout=aiohttp.ClientTimeout(total=gen_timeout)
                     ) as response:
                         response.raise_for_status()
                         data = await response.json()
                         if "response" not in data:
                             raise Exception("Invalid response format from Ollama server")
+                            
+                        # Update the model usage timestamp to keep it hot
+                        self.update_model_usage(model)
+                        
                         return data["response"]
                         
             except aiohttp.ClientConnectorError:
                 last_error = "Could not connect to Ollama server. Make sure Ollama is running and accessible at " + self.base_url
             except aiohttp.ClientResponseError as e:
                 last_error = f"Ollama API error: {e.status} - {e.message}"
-            except aiohttp.ClientTimeout:
+            except asyncio.TimeoutError:
                 last_error = "Request to Ollama server timed out"
             except json.JSONDecodeError:
                 last_error = "Invalid JSON response from Ollama server"
@@ -280,12 +403,12 @@ class OllamaClient(BaseModelClient):
                     break
                     
             if not model_exists:
-                debug_log(f"Model '{model}' not found in available models")
-                # Instead of failing, yield a helpful error message
-                yield f"Model '{model}' not found. Available models include: {', '.join(available_model_names[:5])}"
+                error_msg = f"Model '{model}' not found in available models. Available models include: {', '.join(available_model_names[:5])}"
                 if len(available_model_names) > 5:
-                    yield f" and {len(available_model_names) - 5} more."
-                yield "\n\nPlease try a different model or check your spelling."
+                    error_msg += f" and {len(available_model_names) - 5} more."
+                logger.error(error_msg)
+                # Instead of raising a custom error, yield the message and return
+                yield error_msg
                 return
         except Exception as e:
             debug_log(f"Error checking model availability: {str(e)}")
@@ -316,10 +439,11 @@ class OllamaClient(BaseModelClient):
                                 "stream": False
                             }
                             
+                        test_timeout = self.get_timeout_for_model(model, "test")
                         async with session.post(
                             f"{self.base_url}/api/generate",
                             json=test_payload,
-                            timeout=2
+                            timeout=aiohttp.ClientTimeout(total=test_timeout)
                         ) as response:
                             if response.status != 200:
                                 logger.warning(f"Model test request failed with status {response.status}")
@@ -329,10 +453,11 @@ class OllamaClient(BaseModelClient):
                                 if response.status == 404:
                                     error_text = await response.text()
                                     debug_log(f"404 error details: {error_text}")
-                                    # This is likely a model not found error
-                                    yield f"Error: Model '{model}' not found on the Ollama server."
-                                    yield "\nPlease check if the model name is correct or try pulling it first."
-                                    return
+                                    error_msg = f"Error: Model '{model}' not found on the Ollama server. Please check if the model name is correct or try pulling it first."
+                                    logger.error(error_msg)
+                                    # Instead of raising, yield the error message for user display
+                                    yield error_msg
+                                    return  # End the generation
                                     
                                 raise aiohttp.ClientError("Model not ready")
                     except (aiohttp.ClientError, asyncio.TimeoutError) as e:
@@ -352,10 +477,11 @@ class OllamaClient(BaseModelClient):
                             debug_log(f"Error preparing pull payload: {str(pull_err)}, using default")
                             pull_payload = {"name": "gemma:2b"}  # Safe default
                             
+                        pull_timeout = self.get_timeout_for_model(model, "pull")
                         async with session.post(
                             f"{self.base_url}/api/pull",
                             json=pull_payload,
-                            timeout=60
+                            timeout=aiohttp.ClientTimeout(total=pull_timeout)
                         ) as pull_response:
                             if pull_response.status != 200:
                                 logger.error("Failed to pull model")
@@ -363,12 +489,14 @@ class OllamaClient(BaseModelClient):
                                 self._model_loading = False  # Reset flag on failure
                                 
                                 # Check if this is a 404 Not Found error
-                                if pull_response.status == 404:
-                                    error_text = await pull_response.text()
+                                if response.status == 404:
+                                    error_text = await response.text()
                                     debug_log(f"404 error details: {error_text}")
                                     # This is likely a model not found in registry
-                                    yield f"Error: Model '{model}' not found in the Ollama registry."
-                                    yield "\nPlease check if the model name is correct or try a different model."
+                                    error_msg = f"Error: Model '{model}' not found in the Ollama registry. Please check if the model name is correct or try a different model."
+                                    logger.error(error_msg)
+                                    # Instead of raising a custom error, yield the message and return
+                                    yield error_msg
                                     return
                                     
                                 raise Exception("Failed to pull model")
@@ -404,10 +532,11 @@ class OllamaClient(BaseModelClient):
                         }
                     
                     debug_log(f"Sending request to Ollama API")
+                    gen_timeout = self.get_timeout_for_model(model, "generate")
                     response = await session.post(
                         f"{self.base_url}/api/generate",
                         json=request_payload,
-                        timeout=60  # Longer timeout for actual generation
+                        timeout=aiohttp.ClientTimeout(total=gen_timeout)
                     )
                     response.raise_for_status()
                     debug_log(f"Response status: {response.status}")
@@ -415,8 +544,12 @@ class OllamaClient(BaseModelClient):
                     # Use a simpler async iteration pattern that's less error-prone
                     debug_log("Starting to process response stream")
                     
-                    # Set a flag to track if we've yielded any content
+                    # Update the model usage timestamp to keep it hot
+                    self.update_model_usage(model)
+                    
+                    # Set a flag to track if we've yielded any real content (not loading messages)
                     has_yielded_content = False
+                    has_yielded_real_content = False
                     
                     async for line in response.content:
                         # Check cancellation periodically
@@ -432,17 +565,36 @@ class OllamaClient(BaseModelClient):
                                 if chunk_str.startswith('{') and chunk_str.endswith('}'):
                                     try:
                                         data = json.loads(chunk_str)
-                                        if isinstance(data, dict) and "response" in data:
-                                            response_text = data["response"]
-                                            if response_text:  # Only yield non-empty responses
-                                                has_yielded_content = True
-                                                chunk_length = len(response_text)
-                                                # Only log occasionally to reduce console spam
-                                                if chunk_length % 20 == 0:
-                                                    debug_log(f"Yielding chunk of length: {chunk_length}")
-                                                yield response_text
+                                        if isinstance(data, dict):
+                                            # Check for error in the chunk
+                                            if "error" in data:
+                                                error_msg = data.get("error", "")
+                                                debug_log(f"Ollama API error in chunk: {error_msg}")
+                                                
+                                                # Handle model loading state
+                                                if "loading model" in error_msg.lower():
+                                                    # Yield a user-friendly message and keep trying
+                                                    yield "The model is still loading. Please wait a moment..."
+                                                    has_yielded_content = True  # We did yield something
+                                                    # Add delay before continuing
+                                                    await asyncio.sleep(2)
+                                                    continue
+                                            
+                                            # Process normal response
+                                            if "response" in data:
+                                                response_text = data["response"]
+                                                if response_text:  # Only yield non-empty responses
+                                                    has_yielded_content = True
+                                                    has_yielded_real_content = True  # This is actual model content
+                                                    chunk_length = len(response_text)
+                                                    # Only log occasionally to reduce console spam
+                                                    if chunk_length % 20 == 0:
+                                                        debug_log(f"Yielding chunk of length: {chunk_length}")
+                                                    yield response_text
+                                            else:
+                                                debug_log(f"JSON chunk missing 'response' key: {chunk_str[:100]}")
                                         else:
-                                            debug_log(f"JSON chunk missing 'response' key: {chunk_str[:100]}")
+                                            debug_log(f"JSON chunk is not a dict: {chunk_str[:100]}")
                                     except json.JSONDecodeError:
                                         debug_log(f"JSON decode error for chunk: {chunk_str[:100]}")
                                 else:
@@ -454,9 +606,9 @@ class OllamaClient(BaseModelClient):
                             # Continue instead of breaking to try processing more chunks
                             continue
                     
-                    # If we didn't yield any content, yield a default message
-                    if not has_yielded_content:
-                        debug_log("No content was yielded from stream, providing fallback response")
+                    # If we didn't yield any real content (only loading messages), yield a default message
+                    if not has_yielded_real_content:
+                        debug_log("No real content was yielded from stream, providing fallback response")
                         yield "I'm sorry, but I couldn't generate a response. Please try again or try a different model."
                     
                     logger.info("Streaming completed successfully")
@@ -473,7 +625,7 @@ class OllamaClient(BaseModelClient):
             except aiohttp.ClientResponseError as e:
                 last_error = f"Ollama API error: {e.status} - {e.message}"
                 debug_log(f"ClientResponseError: {last_error}")
-            except aiohttp.ClientTimeout:
+            except asyncio.TimeoutError:
                 last_error = "Request to Ollama server timed out"
                 debug_log(f"ClientTimeout: {last_error}")
             except asyncio.CancelledError:
@@ -507,6 +659,123 @@ class OllamaClient(BaseModelClient):
     def is_loading_model(self) -> bool:
         """Check if Ollama is currently loading a model"""
         return self._model_loading
+    
+    async def preload_model(self, model_id: str) -> bool:
+        """
+        Preload a model to keep it hot/ready for use
+        Returns True if successful, False otherwise
+        """
+        from datetime import datetime
+        import asyncio
+        
+        logger.info(f"Preloading model: {model_id}")
+        
+        # First, check if the model is already preloaded
+        if model_id in self._preloaded_models:
+            # Update timestamp if already preloaded
+            self._preloaded_models[model_id] = datetime.now()
+            logger.info(f"Model {model_id} already preloaded, updated timestamp")
+            return True
+        
+        try:
+            # We'll use a minimal prompt to load the model
+            warm_up_prompt = "hello"
+            
+            # Set model loading state
+            old_loading_state = self._model_loading
+            self._model_loading = True
+            
+            async with aiohttp.ClientSession() as session:
+                # First try pulling the model if needed
+                try:
+                    logger.info(f"Ensuring model {model_id} is pulled")
+                    pull_payload = {"name": model_id}
+                    pull_timeout = self.get_timeout_for_model(model_id, "pull")
+                    async with session.post(
+                        f"{self.base_url}/api/pull",
+                        json=pull_payload,
+                        timeout=aiohttp.ClientTimeout(total=pull_timeout)
+                    ) as pull_response:
+                        # We don't need to process the full pull, just initiate it
+                        if pull_response.status != 200:
+                            logger.warning(f"Pull request for model {model_id} failed with status {pull_response.status}")
+                except Exception as e:
+                    logger.warning(f"Error during model pull check: {str(e)}")
+                
+                # Now send a small generation request to load the model into memory
+                logger.info(f"Sending warm-up request for model {model_id}")
+                gen_timeout = self.get_timeout_for_model(model_id, "load")
+                async with session.post(
+                    f"{self.base_url}/api/generate",
+                    json={
+                        "model": model_id,
+                        "prompt": warm_up_prompt,
+                        "temperature": 0.7,
+                        "stream": False
+                    },
+                    timeout=aiohttp.ClientTimeout(total=gen_timeout)
+                ) as response:
+                    if response.status != 200:
+                        logger.error(f"Failed to preload model {model_id}, status: {response.status}")
+                        self._model_loading = old_loading_state
+                        return False
+                    
+                    # Read the response to ensure the model is fully loaded
+                    await response.json()
+                    
+                    # Update preloaded models with timestamp
+                    self._preloaded_models[model_id] = datetime.now()
+                    logger.info(f"Successfully preloaded model {model_id}")
+                    return True
+        except Exception as e:
+            logger.error(f"Error preloading model {model_id}: {str(e)}")
+            return False
+        finally:
+            # Reset model loading state
+            self._model_loading = old_loading_state
+    
+    def get_preloaded_models(self) -> Dict[str, datetime]:
+        """Return the dict of preloaded models and their last use times"""
+        return self._preloaded_models
+    
+    def update_model_usage(self, model_id: str) -> None:
+        """Update the timestamp for a model that is being used"""
+        if model_id and model_id in self._preloaded_models:
+            from datetime import datetime
+            self._preloaded_models[model_id] = datetime.now()
+            logger.info(f"Updated usage timestamp for model {model_id}")
+    
+    async def release_inactive_models(self, max_inactive_minutes: int = 30) -> List[str]:
+        """
+        Release models that have been inactive for more than the specified time
+        Returns a list of model IDs that were released
+        """
+        from datetime import datetime, timedelta
+        
+        if not self._preloaded_models:
+            return []
+            
+        now = datetime.now()
+        inactive_threshold = timedelta(minutes=max_inactive_minutes)
+        models_to_release = []
+        
+        # Find models that have been inactive for too long
+        for model_id, last_used in list(self._preloaded_models.items()):
+            if now - last_used > inactive_threshold:
+                models_to_release.append(model_id)
+                
+        # Release the models
+        released_models = []
+        for model_id in models_to_release:
+            try:
+                logger.info(f"Releasing inactive model: {model_id} (inactive for {(now - self._preloaded_models[model_id]).total_seconds() / 60:.1f} minutes)")
+                # We don't have an explicit "unload" API in Ollama, but we can remove it from our tracking
+                del self._preloaded_models[model_id]
+                released_models.append(model_id)
+            except Exception as e:
+                logger.error(f"Error releasing model {model_id}: {str(e)}")
+                
+        return released_models
             
     async def get_model_details(self, model_id: str) -> Dict[str, Any]:
         """Get detailed information about a specific Ollama model"""
@@ -516,7 +785,7 @@ class OllamaClient(BaseModelClient):
                 async with session.post(
                     f"{self.base_url}/api/show",
                     json={"name": model_id},
-                    timeout=5
+                    timeout=aiohttp.ClientTimeout(total=5)
                 ) as response:
                     response.raise_for_status()
                     data = await response.json()
@@ -578,7 +847,7 @@ class OllamaClient(BaseModelClient):
                     logger.info(f"Fetching all models from Ollama web: {search_url}")
                     async with session.get(
                         search_url,
-                        timeout=20,  # Longer timeout for comprehensive scrape
+                        timeout=aiohttp.ClientTimeout(total=20),  # Longer timeout for comprehensive scrape
                         headers={"User-Agent": "Mozilla/5.0 (compatible; chat-console/1.0)"}
                     ) as response:
                         if response.status == 200:
@@ -1190,7 +1459,7 @@ class OllamaClient(BaseModelClient):
                 async with session.post(
                     f"{self.base_url}/api/pull",
                     json={"name": model_id},
-                    timeout=3600  # 1 hour timeout for large models
+                    timeout=aiohttp.ClientTimeout(total=3600)  # 1 hour timeout for large models
                 ) as response:
                     response.raise_for_status()
                     async for line in response.content:
@@ -1213,7 +1482,7 @@ class OllamaClient(BaseModelClient):
                 async with session.delete(
                     f"{self.base_url}/api/delete",
                     json={"name": model_id},
-                    timeout=30
+                    timeout=aiohttp.ClientTimeout(total=30)
                 ) as response:
                     response.raise_for_status()
                     logger.info(f"Model {model_id} deleted successfully")
