@@ -17,18 +17,20 @@ logger = logging.getLogger(__name__)
 class ModelSelector(Container):
     """Widget for selecting the AI model to use"""
     
+    # Rams-inspired selector styling
     DEFAULT_CSS = """
     ModelSelector {
         width: 100%;
         height: auto;
-        padding: 0;
-        background: $surface-darken-1;
+        padding: 1;
+        background: #0C0C0C;
+        border: solid #333333 1;
     }
     
     #selector-container {
         width: 100%;
         layout: horizontal;
-        height: 3;
+        height: auto;
         padding: 0;
     }
     
@@ -36,11 +38,17 @@ class ModelSelector(Container):
         width: 30%;
         height: 3;
         margin-right: 1;
+        background: #0C0C0C;
+        color: #E8E8E8;
+        border: solid #333333 1;
     }
     
     #model-select, #custom-model-input {
         width: 1fr;
         height: 3;
+        background: #0C0C0C;
+        color: #E8E8E8;
+        border: solid #333333 1;
     }
 
     #custom-model-input {
@@ -247,6 +255,16 @@ class ModelSelector(Container):
         """Handle select changes"""
         if event.select.id == "provider-select":
             self.selected_provider = event.value
+            
+            # IMPORTANT: Clear any cached client
+            if hasattr(self.app, 'cached_client'):
+                self.app.cached_client = None
+            
+            # Store the selected provider in the app instance for client resolution
+            if hasattr(self.app, 'selected_provider'):
+                self.app.selected_provider = self.selected_provider
+                logger.info(f"Updated app.selected_provider to: {self.selected_provider}")
+                
             # Update model options
             model_select = self.query_one("#model-select", Select)
             model_options = await self._get_model_options(self.selected_provider)
